from datetime import date, datetime, time
import json

import pytest

from attrs import define

import bson
import cbor2
import msgpack
import orjson
import ujson
import yaml
import tomlkit

from msgspec import json as msgspec_json

from django_cattrs_fields.converters import converter
from django_cattrs_fields.converters.bson import serializer as bson_serializer
from django_cattrs_fields.converters.cbor2 import serializer as cbor2_serializer
from django_cattrs_fields.converters.json import serializer as json_serializer
from django_cattrs_fields.converters.msgpack import serializer as msgpack_serializer
from django_cattrs_fields.converters.msgspec import serializer as msgspec_serializer
from django_cattrs_fields.converters.orjson import serializer as orjson_serializer
from django_cattrs_fields.converters.pyyaml import serializer as pyyaml_serializer
from django_cattrs_fields.converters.tomlkit import serializer as tomlkit_serializer
from django_cattrs_fields.converters.ujson import serializer as ujson_serializer
from django_cattrs_fields.fields import DateField, DateTimeField, TimeField
from django_cattrs_fields.utils.timezone import enforce_timezone


@define
class Human:
    birth: DateField
    death: DateTimeField
    work_start: TimeField


@define
class HumanNullable:
    birth: DateField | None
    death: DateTimeField | None
    work_start: TimeField | None


def test_structure():
    d = datetime(year=2080, month=1, day=21)
    h = {
        "birth": date(year=2000, month=5, day=11),
        "death": d,
        "work_start": time(hour=9, minute=0, second=0),
    }

    structure = converter.structure(h, Human)

    h["death"] = enforce_timezone(d)
    obj = Human(**h)

    assert structure == obj
    assert isinstance(structure.birth, date)
    assert isinstance(structure.death, datetime)
    assert isinstance(structure.work_start, time)


@pytest.mark.parametrize("b", [date.today(), None])
@pytest.mark.parametrize("d", [datetime.now(), None])
@pytest.mark.parametrize("t", [time(hour=8), None])
def test_structure_nullable(b, d, t):
    h = {"birth": b, "death": d, "work_start": t}

    structure = converter.structure(h, HumanNullable)

    if h["death"]:
        h["death"] = enforce_timezone(d)
    obj = HumanNullable(**h)

    assert structure == obj


def test_structure_datetime_as_date_date_as_datetime_time_as_str():
    h = {
        "birth": datetime(year=2080, month=1, day=21),
        "death": date(year=2000, month=5, day=11),
        "work_start": time(8).strftime("%H:%M:%S"),
    }

    structure = converter.structure(h, Human)

    assert isinstance(structure.birth, date)
    assert isinstance(structure.death, datetime)
    assert isinstance(structure.work_start, time)


def test_structure_iso_format():
    b = date(year=2000, month=5, day=11)
    d = datetime(year=2080, month=1, day=21)
    t = time(9)
    h = {"birth": b.isoformat(), "death": d.isoformat(), "work_start": t.isoformat()}

    structure = converter.structure(h, Human)

    assert isinstance(structure.birth, date)
    assert isinstance(structure.death, datetime)
    assert isinstance(structure.work_start, time)
    assert structure.birth == b
    assert structure.death == enforce_timezone(d)
    assert structure.work_start == t


def test_unstructure():
    d = datetime(year=2080, month=1, day=21)
    h = {"birth": date(year=2000, month=5, day=11), "death": d, "work_start": time(8)}

    structure = converter.structure(h, Human)

    unstructure = converter.unstructure(structure)
    h["death"] = enforce_timezone(d)

    assert unstructure == h


@pytest.mark.parametrize(
    "b, d, t", [(date.today(), None, None), (None, datetime.now(), None), (None, None, time(7))]
)
def test_unstructure_nullable(b, d, t):
    h = {"birth": b, "death": d, "work_start": t}

    structure = converter.structure(h, HumanNullable)

    unstructure = converter.unstructure(structure)
    if h["death"]:
        h["death"] = enforce_timezone(d)

    assert unstructure == h


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (tomlkit_serializer, tomlkit.dumps),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_dumps(converter, dumps):
    b = date(year=2000, month=5, day=11)
    d = datetime(year=2080, month=1, day=21)
    t = time(5)
    h = {"birth": b, "death": d, "work_start": t}

    structure = converter.structure(h, Human)

    dump = converter.dumps(structure)

    if converter in {
        json_serializer,
        msgpack_serializer,
        ujson_serializer,
        bson_serializer,
    }:
        h["birth"] = b.isoformat()

    h["death"] = enforce_timezone(d)
    if converter in {
        json_serializer,
        msgpack_serializer,
        ujson_serializer,
    }:
        h["death"] = h["death"].isoformat()

    if converter in {
        cbor2_serializer,
        ujson_serializer,
        msgpack_serializer,
        bson_serializer,
        json_serializer,
        pyyaml_serializer,
    }:
        h["work_start"] = h["work_start"].isoformat()

    assert dump == dumps(h)


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (ujson_serializer, ujson.dumps),
    ],
)
@pytest.mark.parametrize(
    "b, d, t",
    [
        (date(year=2000, month=5, day=11), None, None),
        (None, datetime(year=2080, month=1, day=21), None),
        (None, None, time(5, 23)),
    ],
)
def test_dumps_nullable(converter, dumps, b, d, t):
    h = {"birth": b, "death": d, "work_start": t}

    structure = converter.structure(h, HumanNullable)

    dump = converter.dumps(structure)

    if converter in {
        json_serializer,
        msgpack_serializer,
        ujson_serializer,
        bson_serializer,
    }:
        if b:
            h["birth"] = b.isoformat()

    if d:
        h["death"] = enforce_timezone(d)
    if (
        converter
        in {
            json_serializer,
            msgpack_serializer,
            ujson_serializer,
        }
        and d
    ):
        h["death"] = h["death"].isoformat()

    if (
        converter
        in {
            msgpack_serializer,
            ujson_serializer,
            bson_serializer,
            json_serializer,
            pyyaml_serializer,
            cbor2_serializer,
        }
        and t
    ):
        h["work_start"] = h["work_start"].isoformat()

    assert dump == dumps(h)


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (tomlkit_serializer, tomlkit.dumps),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_loads(converter, dumps):
    b = date(year=2000, month=5, day=11)
    d = datetime(year=2080, month=1, day=21)
    t = time(13, 21)
    h = {"birth": b.isoformat(), "death": d.isoformat(), "work_start": t.isoformat()}

    dump = dumps(h)

    load = converter.loads(dump, Human)
    assert load == converter.structure(h, Human)


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (ujson_serializer, ujson.dumps),
    ],
)
@pytest.mark.parametrize(
    "b, d, t",
    [
        (date(year=2000, month=5, day=11), None, None),
        (None, datetime(year=2080, month=1, day=21), None),
        (None, None, time(10, 44)),
    ],
)
def test_loads_nullable(converter, dumps, b, d, t):
    h = {
        "birth": b.isoformat() if b else None,
        "death": d.isoformat() if d else None,
        "work_start": t.isoformat() if t else None,
    }
    hc = {"birth": b if b else None, "death": d if d else None, "work_start": t if t else None}

    dump = dumps(h)

    load = converter.loads(dump, HumanNullable)
    assert load == converter.structure(h, HumanNullable)
    assert load == converter.structure(hc, HumanNullable)


@pytest.mark.parametrize(
    "converter",
    [
        pytest.param(
            bson_serializer, marks=pytest.mark.xfail
        ),  # it seems bson fails to serializer datetime safely
        (cbor2_serializer),
        (json_serializer),
        (msgpack_serializer),
        (msgspec_serializer),
        (orjson_serializer),
        (pyyaml_serializer),
        (tomlkit_serializer),
        (ujson_serializer),
    ],
)
def test_dump_then_load(converter):
    b = date(year=2000, month=5, day=11)
    d = datetime(year=2080, month=1, day=21, hour=2, minute=5)
    t = time(5, 11, 11)
    h = {"birth": b, "death": d, "work_start": t}

    structure = converter.structure(h, Human)

    dump = converter.dumps(structure)
    load = converter.loads(dump, Human)

    h["death"] = enforce_timezone(h["death"])

    assert load.birth == h["birth"]
    assert load.death == h["death"]
    assert load.work_start == h["work_start"]


@pytest.mark.parametrize(
    "converter",
    [
        pytest.param(
            bson_serializer, marks=pytest.mark.xfail
        ),  # it seems bson fails to work datetime safely
        (cbor2_serializer),
        (json_serializer),
        (msgpack_serializer),
        (msgspec_serializer),
        (orjson_serializer),
        (pyyaml_serializer),
        (ujson_serializer),
    ],
)
@pytest.mark.parametrize(
    "b, d, t",
    [
        (date(year=2000, month=5, day=11), None, None),
        (None, datetime(year=2080, month=1, day=21, hour=2, minute=5), None),
        (None, None, time(0, 1, 4)),
    ],
)
def test_dump_then_load_nullable(converter, b, d, t):
    h = {"birth": b, "death": d, "work_start": t}

    structure = converter.structure(h, HumanNullable)

    dump = converter.dumps(structure)
    load = converter.loads(dump, HumanNullable)

    if d:
        h["death"] = enforce_timezone(h["death"])

    assert load.birth == h["birth"]
    assert load.death == h["death"]
    assert load.work_start == h["work_start"]



from typing import Any


from django_cattrs_fields.fields import (
    BooleanField,
)

BOOLEAN_VALUES: dict[Any, bool] = {
    "t": True,
    "y": True,
    "yes": True,
    "true": True,
    "on": True,
    "1": True,
    1: True,
    "f": False,
    "n": False,
    "no": False,
    "false": False,
    "off": False,
    "0": False,
    0: False,
}

__all__ = (
    "boolean_structure",
    "boolean_structure_nullable",
    "boolean_unstructure",
    "BOOLEAN_VALUES",
)


def boolean_structure(val, _) -> BooleanField:
    try:
        val = BOOLEAN_VALUES[val]
    except KeyError as e:
        raise ValueError from e

    return val


def boolean_structure_nullable(val, _) -> BooleanField | None:
    if val is None:
        return None
    return boolean_structure(val, _)


def boolean_unstructure(val: BooleanField | None) -> bool | None:
    return val



import io
import json

import pytest

from django.core.files.uploadedfile import (
    SimpleUploadedFile,
    TemporaryUploadedFile,
    UploadedFile,
    InMemoryUploadedFile,
)

from attrs import define

import bson
import cbor2
import msgpack
import orjson
import ujson
import yaml
import tomlkit

from msgspec import json as msgspec_json

from django_cattrs_fields.converters import converter
from django_cattrs_fields.converters.bson import serializer as bson_serializer
from django_cattrs_fields.converters.cbor2 import serializer as cbor2_serializer
from django_cattrs_fields.converters.json import serializer as json_serializer
from django_cattrs_fields.converters.msgpack import serializer as msgpack_serializer
from django_cattrs_fields.converters.msgspec import serializer as msgspec_serializer
from django_cattrs_fields.converters.orjson import serializer as orjson_serializer
from django_cattrs_fields.converters.pyyaml import serializer as pyyaml_serializer
from django_cattrs_fields.converters.tomlkit import serializer as tomlkit_serializer
from django_cattrs_fields.converters.ujson import serializer as ujson_serializer
from django_cattrs_fields.fields.files import FileField

from tests.books.models import Book


@define
class PDF:
    pdf: FileField


@define
class PDFNullable:
    pdf: FileField | None


@pytest.fixture
def memory_file():
    content = b"Hello, this is an in-memory uploaded file."
    file_buffer = io.BytesIO(content)
    file = InMemoryUploadedFile(
        file=file_buffer,
        field_name="file",
        name="example.txt",
        content_type="text/plain",
        size=len(content),
        charset=None,
    )
    file.seek(0)
    return file


@pytest.fixture
def temp_file():
    file = TemporaryUploadedFile(
        name="example_temp.txt",
        content_type="text/plain",
        size=0,
        charset=None,
    )
    file.write(b"Hello, this is a temporary uploaded file.")
    file.seek(0)
    return file


@pytest.fixture
def simple_file():
    return SimpleUploadedFile(name="test_image.jpeg", content=b"wheeee", content_type="image/jpeg")


def test_structure_uploaded(memory_file, temp_file, simple_file):
    pdf = {"pdf": memory_file}

    structure = converter.structure(pdf, PDF)
    obj = PDF(**pdf)

    assert structure == obj
    assert isinstance(structure.pdf, UploadedFile)
    assert isinstance(structure.pdf, InMemoryUploadedFile)

    pdf = {"pdf": temp_file}

    structure = converter.structure(pdf, PDF)
    obj = PDF(**pdf)

    assert structure == obj
    assert isinstance(structure.pdf, UploadedFile)
    assert isinstance(structure.pdf, TemporaryUploadedFile)

    pdf = {"pdf": simple_file}
    structure = converter.structure(pdf, PDF)
    obj = PDF(**pdf)

    assert structure == obj
    assert isinstance(structure.pdf, UploadedFile)
    assert isinstance(structure.pdf, SimpleUploadedFile)


@pytest.mark.parametrize(
    "memory, temp, simple", [(True, False, False), (False, True, False), (False, False, True)]
)
def test_structure_uploaded_nullable(memory_file, temp_file, simple_file, memory, temp, simple):
    pdf = {"pdf": memory_file if memory else None}

    structure = converter.structure(pdf, PDF)
    obj = PDF(**pdf)

    assert structure == obj
    if memory:
        assert isinstance(structure.pdf, UploadedFile)
        assert isinstance(structure.pdf, InMemoryUploadedFile)
    else:
        assert structure.pdf is None

    pdf = {"pdf": temp_file if temp else None}

    structure = converter.structure(pdf, PDF)
    obj = PDF(**pdf)

    assert structure == obj
    if temp:
        assert isinstance(structure.pdf, UploadedFile)
        assert isinstance(structure.pdf, TemporaryUploadedFile)
    else:
        assert structure.pdf is None

    pdf = {"pdf": simple_file if simple else None}
    structure = converter.structure(pdf, PDF)
    obj = PDF(**pdf)

    assert structure == obj
    if simple:
        assert isinstance(structure.pdf, UploadedFile)
        assert isinstance(structure.pdf, SimpleUploadedFile)
    else:
        assert structure.pdf is None


def test_unstructure_uploaded(memory_file, temp_file, simple_file):
    pdf = {"pdf": memory_file}
    structure = converter.structure(pdf, PDF)

    unstructure = converter.unstructure(structure)
    assert unstructure == pdf
    assert isinstance(structure.pdf, InMemoryUploadedFile)

    pdf = {"pdf": temp_file}
    structure = converter.structure(pdf, PDF)

    unstructure = converter.unstructure(structure)
    assert unstructure == pdf
    assert isinstance(structure.pdf, TemporaryUploadedFile)

    pdf = {"pdf": simple_file}
    structure = converter.structure(pdf, PDF)

    unstructure = converter.unstructure(structure)
    assert unstructure == pdf
    assert isinstance(structure.pdf, SimpleUploadedFile)


@pytest.mark.parametrize(
    "memory, temp, simple", [(True, False, False), (False, True, False), (False, False, True)]
)
def test_unstructure_uploaded_nullable(memory_file, temp_file, simple_file, memory, temp, simple):
    pdf = {"pdf": memory_file if memory else None}

    structure = converter.structure(pdf, PDF)
    unstructure = converter.unstructure(structure)

    assert unstructure == pdf
    if memory:
        assert isinstance(unstructure["pdf"], UploadedFile)
        assert isinstance(unstructure["pdf"], InMemoryUploadedFile)
    else:
        assert unstructure["pdf"] is None

    pdf = {"pdf": temp_file if temp else None}

    structure = converter.structure(pdf, PDF)
    unstructure = converter.unstructure(structure)

    assert unstructure == pdf
    if temp:
        assert isinstance(unstructure["pdf"], UploadedFile)
        assert isinstance(unstructure["pdf"], TemporaryUploadedFile)
    else:
        assert unstructure["pdf"] is None

    pdf = {"pdf": simple_file if simple else None}
    structure = converter.structure(pdf, PDF)
    unstructure = converter.unstructure(structure)

    assert unstructure == pdf
    if simple:
        assert isinstance(unstructure["pdf"], UploadedFile)
        assert isinstance(unstructure["pdf"], SimpleUploadedFile)
    else:
        assert unstructure["pdf"] is None


def test_structure_field_file(simple_file, db):
    b = Book.objects.create(pdf=simple_file)
    s = {"pdf": b.pdf}

    structure = converter.structure(s, PDF)
    assert isinstance(structure.pdf, str)
    assert structure.pdf == b.pdf.url


@pytest.mark.parametrize("simple", [True, False])
def test_structure_field_file_nullable(simple_file, db, simple):
    b = Book.objects.create(pdf=simple_file if simple else None)
    s = {"pdf": b.pdf}

    structure = converter.structure(s, PDFNullable)
    if simple:
        assert isinstance(structure.pdf, str)
        assert structure.pdf == b.pdf.url
    else:
        assert structure.pdf is None


def test_unstructure_field_file(simple_file, db):
    b = Book.objects.create(pdf=simple_file)
    s = {"pdf": b.pdf}
    structure = converter.structure(s, PDF)

    unstructure = converter.unstructure(structure)

    assert isinstance(unstructure["pdf"], str)
    assert unstructure["pdf"] == b.pdf.url


@pytest.mark.parametrize("simple", [True, False])
def test_unstructure_field_file_nullable(simple_file, db, simple):
    b = Book.objects.create(pdf=simple_file if simple else None)
    s = {"pdf": b.pdf}
    structure = converter.structure(s, PDFNullable)

    unstructure = converter.unstructure(structure)

    if simple:
        assert isinstance(unstructure["pdf"], str)
        assert unstructure["pdf"] == b.pdf.url
    else:
        assert unstructure["pdf"] is None


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (tomlkit_serializer, tomlkit.dumps),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_dumps(db, converter, dumps, simple_file, memory_file, temp_file):
    b = Book.objects.create(pdf=simple_file)

    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDF)
    dump = converter.dumps(structure)
    pdf["pdf"] = pdf["pdf"].url
    assert dump == dumps(pdf)

    b = Book.objects.create(pdf=memory_file)
    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDF)
    dump = converter.dumps(structure)
    pdf["pdf"] = pdf["pdf"].url
    assert dump == dumps(pdf)

    b = Book.objects.create(pdf=temp_file)
    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDF)
    dump = converter.dumps(structure)
    pdf["pdf"] = pdf["pdf"].url
    assert dump == dumps(pdf)


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (ujson_serializer, ujson.dumps),
    ],
)
@pytest.mark.parametrize(
    "memory, temp, simple", [(True, False, False), (False, True, False), (False, False, True)]
)
def test_dumps_nullable(
    memory_file, temp_file, simple_file, memory, temp, simple, converter, dumps, db
):
    b = Book.objects.create(pdf=simple_file if simple else None)

    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDFNullable)
    dump = converter.dumps(structure)
    pdf["pdf"] = pdf["pdf"].url if simple else None

    assert dump == dumps(pdf)

    b = Book.objects.create(pdf=memory_file if memory else None)
    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDFNullable)
    dump = converter.dumps(structure)
    pdf["pdf"] = pdf["pdf"].url if memory else None

    assert dump == dumps(pdf)

    b = Book.objects.create(pdf=temp_file if temp else None)
    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDFNullable)
    dump = converter.dumps(structure)
    pdf["pdf"] = pdf["pdf"].url if temp else None

    assert dump == dumps(pdf)


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (tomlkit_serializer, tomlkit.dumps),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_loads(db, converter, dumps, simple_file, memory_file, temp_file):
    b = Book.objects.create(pdf=simple_file)
    pdf = {"pdf": b.pdf.url}

    dump = dumps(pdf)

    assert converter.loads(dump, PDF) == PDF(**pdf)

    b = Book.objects.create(pdf=memory_file)
    pdf = {"pdf": b.pdf.url}

    dump = dumps(pdf)
    assert converter.loads(dump, PDF) == PDF(**pdf)

    b = Book.objects.create(pdf=temp_file)
    pdf = {"pdf": b.pdf.url}

    dump = dumps(pdf)

    assert converter.loads(dump, PDF) == PDF(**pdf)


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (ujson_serializer, ujson.dumps),
    ],
)
@pytest.mark.parametrize(
    "memory, temp, simple", [(True, False, False), (False, True, False), (False, False, True)]
)
def test_loads_nullable(
    db, converter, dumps, simple_file, memory_file, temp_file, memory, temp, simple
):
    b = Book.objects.create(pdf=simple_file if simple else None)
    pdf = {"pdf": b.pdf.url if simple else None}

    dump = dumps(pdf)

    assert converter.loads(dump, PDFNullable) == PDFNullable(**pdf)

    b = Book.objects.create(pdf=memory_file if memory else None)
    pdf = {"pdf": b.pdf.url if memory else None}

    dump = dumps(pdf)
    assert converter.loads(dump, PDFNullable) == PDFNullable(**pdf)

    b = Book.objects.create(pdf=temp_file if temp else None)
    pdf = {"pdf": b.pdf.url if temp else None}

    dump = dumps(pdf)

    assert converter.loads(dump, PDFNullable) == PDFNullable(**pdf)


@pytest.mark.parametrize(
    "converter",
    [
        (bson_serializer),
        (cbor2_serializer),
        (json_serializer),
        (msgpack_serializer),
        (msgspec_serializer),
        (orjson_serializer),
        (pyyaml_serializer),
        (tomlkit_serializer),
        (ujson_serializer),
    ],
)
def test_dumps_then_loads(db, converter, simple_file, memory_file, temp_file):
    b = Book.objects.create(pdf=simple_file)
    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDF)
    dump = converter.dumps(structure)
    pdf["pdf"] = pdf["pdf"].url
    obj = PDF(**pdf)

    assert converter.loads(dump, PDF) == obj

    b = Book.objects.create(pdf=memory_file)
    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDF)
    dump = converter.dumps(structure)
    pdf["pdf"] = pdf["pdf"].url
    obj = PDF(**pdf)

    assert converter.loads(dump, PDF) == obj

    b = Book.objects.create(pdf=temp_file)
    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDF)
    dump = converter.dumps(structure)
    pdf["pdf"] = pdf["pdf"].url
    obj = PDF(**pdf)

    assert converter.loads(dump, PDF) == obj


@pytest.mark.parametrize(
    "converter",
    [
        (bson_serializer),
        (cbor2_serializer),
        (json_serializer),
        (msgpack_serializer),
        (msgspec_serializer),
        (orjson_serializer),
        (pyyaml_serializer),
        (ujson_serializer),
    ],
)
@pytest.mark.parametrize(
    "memory, temp, simple", [(True, False, False), (False, True, False), (False, False, True)]
)
def test_dumps_then_loads_nullable(
    db, converter, simple_file, memory_file, temp_file, memory, temp, simple
):
    b = Book.objects.create(pdf=simple_file if simple else None)
    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDFNullable)
    dump = converter.dumps(structure)
    if simple:
        pdf["pdf"] = pdf["pdf"].url
    obj = PDFNullable(**pdf)

    assert converter.loads(dump, PDFNullable) == obj

    b = Book.objects.create(pdf=memory_file if memory else None)
    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDFNullable)
    dump = converter.dumps(structure)
    if memory:
        pdf["pdf"] = pdf["pdf"].url
    obj = PDFNullable(**pdf)

    assert converter.loads(dump, PDFNullable) == obj

    b = Book.objects.create(pdf=temp_file if temp else None)
    pdf = {"pdf": b.pdf}

    structure = converter.structure(pdf, PDFNullable)
    dump = converter.dumps(structure)
    if temp:
        pdf["pdf"] = pdf["pdf"].url
    obj = PDFNullable(**pdf)

    assert converter.loads(dump, PDFNullable) == obj



from django.core.files.uploadedfile import (
    InMemoryUploadedFile,
    SimpleUploadedFile,
    TemporaryUploadedFile,
    UploadedFile,
)
from django.db.models.fields.files import FieldFile
from django.utils.translation import gettext_lazy as _

from django_cattrs_fields.fields.files import FileField

__all__ = (
    "file_structure",
    "file_structure_nullable",
    "file_unstructure",
)


def file_structure(
    val: InMemoryUploadedFile | TemporaryUploadedFile | SimpleUploadedFile | FieldFile | str, _t
) -> FileField | str:
    # handling file uploaded by client
    if isinstance(val, UploadedFile):
        if not hasattr(val, "name") and not hasattr(val, "size"):
            raise ValueError(_("No file was submitted. Check the encoding type on the form."))
        if not val.name:
            raise ValueError(_("No filename could be determined."))

        # this is either a `InMemoryUploadedFile` or `TemporaryUploadedFile` or a custom object
        return val

    # GET request, return the file url to client
    elif isinstance(val, FieldFile):
        return val.url

    # if using this field as a client, and we call some API, we get the url back as a string
    return val


def file_structure_nullable(
    val: InMemoryUploadedFile | TemporaryUploadedFile | SimpleUploadedFile | FieldFile | str | None,
    _t,
) -> FileField | str | None:
    if not val:
        return None
    return file_structure(val, _t)


def file_unstructure(
    val: FileField | FieldFile | str | None,
) -> InMemoryUploadedFile | TemporaryUploadedFile | str | None:
    if not val:
        return None
    # handling file uploaded by client
    if isinstance(val, UploadedFile):
        return val

    # normally a structured data doesn't have FieldFile,
    # but just in case user wants to do something manual
    elif isinstance(val, FieldFile):
        return val.url

    # url
    return val



import datetime
import uuid
from decimal import Decimal
from typing import final

from attrs import frozen


@frozen
class Params:
    """param class for passing extra data to fields

    Attributes
    ----------
    decimal_max_digits : (for DecimalField only) maximum number of digits allowed in the number.

    decimal_places : (for DecimalField only) the number of decimal places to store with the number.
    """

    decimal_max_digits: int | None = None  # for DecimalField only
    decimal_places: int | None = None  # for DecimalField only


type BooleanField = bool

type CharField = str

type EmailField = str

type SlugField = str

type URLField = str

type UUIDField = uuid.UUID

type IntegerField = int

type DecimalField = Decimal

type FloatField = float

type DateField = datetime.date

type DateTimeField = datetime.datetime

type TimeField = datetime.time


@final
class EmptyField:
    """used for creating a type checker friendly sentinel for empty values
    empty values are omitted when unstructuring, suitable for PATCH requests.
    """

    __slots__ = ()

    def __repr__(self) -> str:
        return "Empty"


Empty = EmptyField()



SECRET_KEY = "django_tests_secret_key"  # noqa: S105
PASSWORD_HASHERS = ("django.contrib.auth.hashers.MD5PasswordHasher",)

# Include `django.contrib.auth` and `django.contrib.contenttypes` for mypy /
# django-stubs.

# See:
# - https://github.com/typeddjango/django-stubs/issues/318
# - https://github.com/typeddjango/django-stubs/issues/534
INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "tests.books",
]

USE_TZ = True

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

ROOT_URLCONF = "tests.conf.urls"



from cattrs.converters import Converter

from .register_hooks import register_unstructure_hooks, register_structure_hooks

converter = Converter()


register_structure_hooks(converter)
register_unstructure_hooks(converter)



import decimal


def quantize(value: decimal.Decimal, attrs: dict):
    """
    Quantize the decimal value to the configured precision.
    """
    if attrs["decimal_places"] is None:
        return value

    context = decimal.getcontext().copy()
    if attrs["max_digits"] is not None:
        context.prec = attrs["max_digits"]
    return value.quantize(decimal.Decimal(".1") ** attrs["decimal_places"], context=context)



from cattrs.preconf.msgspec import make_converter

from .register_hooks import (
    register_structure_hooks,
    register_all_unstructure_hooks,
)

serializer = make_converter()

register_structure_hooks(serializer)
register_all_unstructure_hooks(serializer)

__all__ = ("serializer",)



import json

import pytest

from attrs import define

import bson
import cbor2
import msgpack
import orjson
import ujson
import yaml
import tomlkit

from msgspec import json as msgspec_json

from django_cattrs_fields.converters import converter
from django_cattrs_fields.converters.bson import serializer as bson_serializer
from django_cattrs_fields.converters.cbor2 import serializer as cbor2_serializer
from django_cattrs_fields.converters.json import serializer as json_serializer
from django_cattrs_fields.converters.msgpack import serializer as msgpack_serializer
from django_cattrs_fields.converters.msgspec import serializer as msgspec_serializer
from django_cattrs_fields.converters.orjson import serializer as orjson_serializer
from django_cattrs_fields.converters.pyyaml import serializer as pyyaml_serializer
from django_cattrs_fields.converters.tomlkit import serializer as tomlkit_serializer
from django_cattrs_fields.converters.ujson import serializer as ujson_serializer
from django_cattrs_fields.fields import BooleanField


@define
class Graduation:
    graduated: BooleanField


@define
class GraduationNullable:
    graduated: BooleanField | None


@pytest.mark.parametrize("b", [True, False])
def test_structure(b):
    g = {"graduated": b}

    structure = converter.structure(g, Graduation)
    obj = Graduation(graduated=b)

    assert structure == obj
    assert isinstance(structure.graduated, bool)


def test_structure_nullable():
    g = {"graduated": None}

    structure = converter.structure(g, GraduationNullable)
    obj = GraduationNullable(graduated=None)

    assert structure == obj
    assert structure.graduated is None

    g2 = {"graduated": True}
    structure = converter.structure(g2, GraduationNullable)
    obj = GraduationNullable(graduated=True)

    assert structure == obj


def test_structure_null_with_not_nullable_raises():
    g = {"graduated": None}

    with pytest.RaisesGroup(ValueError):
        converter.structure(g, Graduation)


@pytest.mark.parametrize("b", [True, False])
def test_unstructure(b):
    g = {"graduated": b}
    structure = converter.structure(g, Graduation)

    unstructure = converter.unstructure(structure)

    assert unstructure == g

    assert isinstance(unstructure["graduated"], bool)


def test_unstructure_nullable():
    g = {"graduated": None}
    structure = converter.structure(g, GraduationNullable)

    unstructure = converter.unstructure(structure)

    assert unstructure == g

    assert unstructure["graduated"] is None


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (tomlkit_serializer, tomlkit.dumps),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_dumps(converter, dumps):
    g = {"graduated": True}
    structure = converter.structure(g, Graduation)

    dump = converter.dumps(structure)

    assert dump == dumps(g)


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_dumps_null(converter, dumps):
    g = {"graduated": None}
    structure = converter.structure(g, GraduationNullable)

    dump = converter.dumps(structure)

    assert dump == dumps(g)


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (tomlkit_serializer, tomlkit.dumps),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_loads(converter, dumps):
    g = {"graduated": True}
    dump = dumps(g)

    x = converter.loads(dump, Graduation)

    assert x == converter.structure(g, Graduation)


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_loads_null(converter, dumps):
    g = {"graduated": None}
    dump = dumps(g)

    x = converter.loads(dump, GraduationNullable)

    assert x == converter.structure(g, GraduationNullable)


@pytest.mark.parametrize(
    "converter",
    [
        (bson_serializer),
        (cbor2_serializer),
        (json_serializer),
        (msgpack_serializer),
        (msgspec_serializer),
        (orjson_serializer),
        (pyyaml_serializer),
        (tomlkit_serializer),
        (ujson_serializer),
    ],
)
def test_dump_then_load(converter):
    g = {"graduated": True}
    structure = converter.structure(g, Graduation)

    dump = converter.dumps(structure)
    load = converter.loads(dump, dict)

    assert load == g


@pytest.mark.parametrize(
    "converter",
    [
        (bson_serializer),
        (cbor2_serializer),
        (json_serializer),
        (msgpack_serializer),
        (msgspec_serializer),
        (orjson_serializer),
        (pyyaml_serializer),
        (ujson_serializer),
    ],
)
def test_dump_then_load_null(converter):
    g = {"graduated": None}
    structure = converter.structure(g, GraduationNullable)

    dump = converter.dumps(structure)
    load = converter.loads(dump, dict)

    assert load == g



import datetime

from django.utils import formats
from django.utils.dateparse import parse_date, parse_datetime
from django.utils.translation import gettext_lazy as _

from django_cattrs_fields.fields import DateField, DateTimeField, TimeField
from django_cattrs_fields.utils.timezone import enforce_timezone
from django_cattrs_fields.validators import forbid_falsy_values_validator


__all__ = (
    "date_structure",
    "date_structure_nullable",
    "date_unstructure",
    "datetime_structure",
    "datetime_structure_nullable",
    "datetime_unstructure",
    "time_structure",
    "time_structure_nullable",
    "time_unstructure",
    "time_unstructure_str",
)

# Date hooks


def date_structure(val, _) -> DateField:
    forbid_falsy_values_validator(val)

    if isinstance(val, datetime.date):
        return val
    elif isinstance(val, datetime.datetime):
        return val.date()
    else:
        try:
            parsed = parse_date(val)
            if parsed is not None:
                return parsed

        except ValueError:
            raise ValueError(
                f"“{val}” value has the correct format (YYYY-MM-DD) but it is an invalid date."
            )
        raise ValueError(
            f"“{val}” value has an invalid date format. It must be in YYYY-MM-DD format."
        )


def date_structure_nullable(val, _) -> DateField | None:
    if val is None:
        return None
    return date_structure(val, _)


def date_unstructure(val: DateField | None) -> datetime.date | None:
    if val is None:
        return None
    return datetime.date(val.year, val.month, val.day)


# DateTime hooks


def datetime_structure(val, _) -> DateTimeField:
    forbid_falsy_values_validator(val)

    if isinstance(val, datetime.datetime):
        return enforce_timezone(val)
    elif isinstance(val, datetime.date):
        return enforce_timezone(datetime.datetime(val.year, val.month, val.day))

    try:
        parsed = parse_datetime(val)
        if parsed is not None:
            return enforce_timezone(parsed)
    except ValueError:
        raise ValueError(
            f"“{val}s” value has the correct format "
            "(YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ]) "
            "but it is an invalid date/time."
        )

    raise ValueError(
        f"“{val}s” value has an invalid format. It must be in "
        "YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ] format."
    )


def datetime_structure_nullable(val, _) -> DateTimeField | None:
    if val is None:
        return None
    return datetime_structure(val, _)


def datetime_unstructure(val: DateTimeField | None) -> datetime.datetime | None:
    if val is None:
        return None

    return val


# Time hooks


def time_structure(val, _t) -> TimeField:
    forbid_falsy_values_validator(val)

    if isinstance(val, datetime.time):
        return val
    for format in formats.get_format_lazy("TIME_INPUT_FORMATS"):
        try:
            return datetime.datetime.strptime(val, format).time()
        except (ValueError, TypeError):
            continue
    raise ValueError(_("Enter a valid time."))


def time_structure_nullable(val, _) -> TimeField | None:
    if val is None:
        return None
    return time_structure(val, _)


def time_unstructure(val: TimeField | None) -> datetime.time | None:
    if val is None:
        return None

    return val


def time_unstructure_str(val: TimeField | None) -> str | None:
    if val is None:
        return None
    return val.isoformat()



from typing import Union

from django.core.files.uploadedfile import (
    InMemoryUploadedFile,
    SimpleUploadedFile,
    TemporaryUploadedFile,
)

type FileField = Union[InMemoryUploadedFile, TemporaryUploadedFile, SimpleUploadedFile, str]



from collections.abc import Callable
from typing import Any

from cattrs.converters import Converter
from cattrs.gen import make_dict_unstructure_fn

from django_cattrs_fields.fields import (
    BooleanField,
    CharField,
    DateField,
    DateTimeField,
    DecimalField,
    Empty,
    EmptyField,
    FloatField,
    IntegerField,
    SlugField,
    TimeField,
    URLField,
    UUIDField,
)
from django_cattrs_fields.fields.files import FileField
from django_cattrs_fields.hooks.bool_hooks import (
    boolean_structure,
    boolean_structure_nullable,
    boolean_unstructure,
)
from django_cattrs_fields.hooks.char_hooks import (
    char_structure,
    char_structure_nullable,
    char_unstructure,
    email_structure,
    email_structure_nullable,
    email_unstructure,
    slug_structure,
    slug_structure_nullable,
    slug_unstructure,
    url_structure,
    url_structure_nullable,
    url_unstructure,
    uuid_structure,
    uuid_structure_nullable,
    uuid_unstructure,
)
from django_cattrs_fields.hooks.date_hooks import (
    date_structure,
    date_structure_nullable,
    date_unstructure,
    datetime_structure,
    datetime_structure_nullable,
    datetime_unstructure,
    time_structure,
    time_structure_nullable,
    time_unstructure,
)
from django_cattrs_fields.hooks.file_hooks import (
    file_structure,
    file_structure_nullable,
    file_unstructure,
)
from django_cattrs_fields.hooks.number_hooks import (
    decimal_structure,
    decimal_structure_nullable,
    decimal_unstructure,
    float_structure,
    float_structure_nullable,
    float_unstructure,
    integer_structure,
    integer_structure_nullable,
    integer_unstructure,
)


def skip_empty(cls: Any, converter: Converter) -> Callable[[Any], dict[str, Any]]:
    fn = make_dict_unstructure_fn(cls, converter)

    def unstructure(obj: Any) -> dict[str, Any]:
        data = fn(obj)
        return {k: v for k, v in data.items() if v is not Empty}

    return unstructure


## Structure


def empty_structure(val, _) -> EmptyField | Any:
    return val


# bool


def empty_bool_structure(val, _) -> EmptyField | BooleanField:
    if val is Empty:
        return Empty
    return boolean_structure(val, _)


def empty_bool_structure_nullable(val, _) -> EmptyField | BooleanField | None:
    if val is Empty:
        return Empty
    return boolean_structure_nullable(val, _)


# char


def empty_char_structure(val, _) -> EmptyField | CharField:
    if val is Empty:
        return Empty
    return char_structure(val, _)


def empty_char_structure_nullable(val, _) -> EmptyField | CharField | None:
    if val is Empty:
        return Empty
    return char_structure_nullable(val, _)


# email


def empty_email_structure(val, _) -> EmptyField | CharField:
    if val is Empty:
        return Empty
    return email_structure(val, _)


def empty_email_structure_nullable(val, _) -> EmptyField | CharField | None:
    if val is Empty:
        return Empty
    return email_structure_nullable(val, _)


# slug


def empty_slug_structure(val, _) -> EmptyField | SlugField:
    if val is Empty:
        return Empty
    return slug_structure(val, _)


def empty_slug_structure_nullable(val, _) -> EmptyField | SlugField | None:
    if val is Empty:
        return Empty
    return slug_structure_nullable(val, _)


# url


def empty_url_structure(val, _) -> EmptyField | URLField:
    if val is Empty:
        return Empty
    return url_structure(val, _)


def empty_url_structure_nullable(val, _) -> EmptyField | URLField | None:
    if val is Empty:
        return Empty
    return url_structure_nullable(val, _)


# uuid


def empty_uuid_structure(val, _) -> EmptyField | UUIDField:
    if val is Empty:
        return Empty
    return uuid_structure(val, _)


def empty_uuid_structure_nullable(val, _) -> EmptyField | UUIDField | None:
    if val is Empty:
        return Empty
    return uuid_structure_nullable(val, _)


# int


def empty_integer_structure(val, _) -> EmptyField | IntegerField:
    if val is Empty:
        return Empty
    return integer_structure(val, _)


def empty_integer_structure_nullable(val, _) -> EmptyField | IntegerField | None:
    if val is Empty:
        return Empty
    return integer_structure_nullable(val, _)


# decimal


def empty_decimal_structure(val, _) -> EmptyField | DecimalField:
    if val is Empty:
        return Empty
    return decimal_structure(val, _)


def empty_decimal_structure_nullable(val, _) -> EmptyField | DecimalField | None:
    if val is Empty:
        return Empty
    return decimal_structure_nullable(val, _)


# float


def empty_float_structure(val, _) -> EmptyField | FloatField:
    if val is Empty:
        return Empty
    return float_structure(val, _)


def empty_float_structure_nullable(val, _) -> EmptyField | FloatField | None:
    if val is Empty:
        return Empty
    return float_structure_nullable(val, _)


# date


def empty_date_structure(val, _) -> EmptyField | DateField:
    if val is Empty:
        return Empty
    return date_structure(val, _)


def empty_date_structure_nullable(val, _) -> EmptyField | DateField | None:
    if val is Empty:
        return Empty
    return date_structure_nullable(val, _)


# datetime


def empty_datetime_structure(val, _) -> EmptyField | DateTimeField:
    if val is Empty:
        return Empty
    return datetime_structure(val, _)


def empty_datetime_structure_nullable(val, _) -> EmptyField | DateTimeField | None:
    if val is Empty:
        return Empty
    return datetime_structure_nullable(val, _)


# time


def empty_time_structure(val, _) -> EmptyField | TimeField:
    if val is Empty:
        return Empty
    return time_structure(val, _)


def empty_time_structure_nullable(val, _) -> EmptyField | TimeField | None:
    if val is Empty:
        return Empty
    return time_structure_nullable(val, _)


# file


def empty_file_structure(val, _) -> EmptyField | FileField:
    if val is Empty:
        return Empty
    return file_structure(val, _)


def empty_file_structure_nullable(val, _) -> EmptyField | FileField | None:
    if val is Empty:
        return Empty
    return file_structure_nullable(val, _)


## Unstructure


def empty_unstructure(val: EmptyField | Any) -> EmptyField | Any:
    return val


# bool


def empty_bool_unstructure(val) -> EmptyField | BooleanField | None:
    if val is Empty:
        return Empty
    return boolean_unstructure(val)


# char


def empty_char_unstructure(val) -> EmptyField | CharField | None:
    if val is Empty:
        return Empty
    return char_unstructure(val)


# email


def empty_email_unstructure(val) -> EmptyField | CharField | None:
    if val is Empty:
        return Empty
    return email_unstructure(val)


# slug


def empty_slug_unstructure(val) -> EmptyField | SlugField | None:
    if val is Empty:
        return Empty
    return slug_unstructure(val)


# url


def empty_url_unstructure(val) -> EmptyField | URLField | None:
    if val is Empty:
        return Empty
    return url_unstructure(val)


# uuid


def empty_uuid_unstructure(val) -> EmptyField | UUIDField | None:
    if val is Empty:
        return Empty
    return uuid_unstructure(val)


# int


def empty_integer_unstructure(val) -> EmptyField | IntegerField | None:
    if val is Empty:
        return Empty
    return integer_unstructure(val)


# decimal


def empty_decimal_unstructure(val) -> EmptyField | DecimalField | None:
    if val is Empty:
        return Empty
    return decimal_unstructure(val)


# float


def empty_float_unstructure(val) -> EmptyField | FloatField | None:
    if val is Empty:
        return Empty
    return float_unstructure(val)


# date


def empty_date_unstructure(val) -> EmptyField | DateField | None:
    if val is Empty:
        return Empty
    return date_unstructure(val)


# datetime


def empty_datetime_unstructure(val) -> EmptyField | DateTimeField | None:
    if val is Empty:
        return Empty
    return datetime_unstructure(val)


# time


def empty_time_unstructure(val) -> EmptyField | TimeField | None:
    if val is Empty:
        return Empty
    return time_unstructure(val)


# file


def empty_file_unstructure(val) -> EmptyField | FileField | None:
    if val is Empty:
        return Empty
    return file_unstructure(val)



from typing import Union

from django.conf import settings

from cattrs.preconf.json import make_converter

from django_cattrs_fields.fields import DateField, DateTimeField, DecimalField, TimeField, UUIDField
from django_cattrs_fields.hooks.date_hooks import time_unstructure_str
from django_cattrs_fields.hooks.number_hooks import decimal_unstructure_str

from .register_hooks import (
    register_structure_hooks,
    register_unstructure_hooks,
)

serializer = make_converter()


register_structure_hooks(serializer)
register_unstructure_hooks(serializer)

if getattr(settings, "DCF_SERIALIZER_HOOKS", True):
    serializer.register_unstructure_hook(UUIDField, lambda x: str(x))
    serializer.register_unstructure_hook(Union[UUIDField, None], lambda x: str(x) if x else None)

    serializer.register_unstructure_hook(DateField, lambda x: x.isoformat())
    serializer.register_unstructure_hook(
        Union[DateField, None], lambda x: x.isoformat() if x else None
    )
    serializer.register_unstructure_hook(DateTimeField, lambda x: x.isoformat())
    serializer.register_unstructure_hook(
        Union[DateTimeField, None], lambda x: x.isoformat() if x else None
    )
    serializer.register_unstructure_hook(DecimalField, decimal_unstructure_str)
    serializer.register_unstructure_hook(Union[DecimalField, None], decimal_unstructure_str)
    serializer.register_unstructure_hook(TimeField, time_unstructure_str)
    serializer.register_unstructure_hook(Union[TimeField, None], time_unstructure_str)

__all__ = ("serializer",)



import json
import uuid

import pytest

from attrs import define

import bson
import cbor2
import msgpack
import orjson
import ujson
import yaml
import tomlkit

from msgspec import json as msgspec_json

from django_cattrs_fields.converters import converter
from django_cattrs_fields.converters.bson import serializer as bson_serializer
from django_cattrs_fields.converters.cbor2 import serializer as cbor2_serializer
from django_cattrs_fields.converters.json import serializer as json_serializer
from django_cattrs_fields.converters.msgpack import serializer as msgpack_serializer
from django_cattrs_fields.converters.msgspec import serializer as msgspec_serializer
from django_cattrs_fields.converters.orjson import serializer as orjson_serializer
from django_cattrs_fields.converters.pyyaml import serializer as pyyaml_serializer
from django_cattrs_fields.converters.tomlkit import serializer as tomlkit_serializer
from django_cattrs_fields.converters.ujson import serializer as ujson_serializer
from django_cattrs_fields.fields import (
    CharField,
    EmailField,
    SlugField,
    URLField,
    UUIDField,
)


@define
class Worker:
    name: CharField
    email: EmailField
    slug: SlugField
    website: URLField
    unique_id: UUIDField


@define
class WorkerNullable:
    name: CharField | None
    email: EmailField | None
    slug: SlugField | None
    website: URLField | None
    unique_id: UUIDField | None


def test_structure():
    w = {
        "name": "bob",
        "email": "bob@email.com",
        "slug": "bo-b",
        "website": "https://bob.com",
        "unique_id": uuid.uuid4(),
    }

    structure = converter.structure(w, Worker)

    obj = Worker(**w)

    assert structure == obj
    assert isinstance(structure.name, str)
    assert isinstance(structure.email, str)
    assert isinstance(structure.slug, str)
    assert isinstance(structure.website, str)
    assert isinstance(structure.unique_id, uuid.UUID)


def test_structure_invalid_name():
    w = {
        "name": 1,
        "email": "bob@email.com",
        "slug": "bo-b",
        "website": "https://bob.com",
        "unique_id": uuid.uuid4(),
    }

    with pytest.RaisesGroup(ValueError) as exp:
        converter.structure(w, Worker)
    assert str(exp.value.exceptions[0]) == "value should be a string object"


def test_structure_invalid_email():
    w = {
        "name": "bob",
        "email": "bob@email",
        "slug": "bo-b",
        "website": "https://bob.com",
        "unique_id": uuid.uuid4(),
    }

    with pytest.RaisesGroup(ValueError) as exp:
        converter.structure(w, Worker)
    assert str(exp.value.exceptions[0]) == "Enter a valid email address."


def test_structure_invalid_slug():
    w = {
        "name": "bob",
        "email": "bob@email.com",
        "slug": "bo b",
        "website": "https://bob.com",
        "unique_id": uuid.uuid4(),
    }

    with pytest.RaisesGroup(ValueError) as exp:
        converter.structure(w, Worker)
    assert (
        str(exp.value.exceptions[0])
        == "Enter a valid “slug” consisting of Unicode letters, numbers, underscores, or hyphens."  # noqa: E501
    )


def test_structure_invalid_website():
    w = {
        "name": "bob",
        "email": "bob@email.com",
        "slug": "bo-b",
        "website": "://bob.com",
        "unique_id": uuid.uuid4(),
    }

    with pytest.RaisesGroup(ValueError) as exp:
        converter.structure(w, Worker)
    assert str(exp.value.exceptions[0]) == "Enter a valid URL."


def test_structure_invalid_uuid():
    w = {
        "name": "bob",
        "email": "bob@email.com",
        "slug": "bo-b",
        "website": "https://bob.com",
        "unique_id": "sfa",
    }

    with pytest.RaisesGroup(ValueError) as exp:
        converter.structure(w, Worker)
    assert str(exp.value.exceptions[0]) == "badly formed hexadecimal UUID string"


def test_structure_invalid_null():
    w = {
        "name": None,
        "email": None,
        "slug": None,
        "website": None,
        "unique_id": None,
    }
    with pytest.RaisesGroup(ValueError, ValueError, ValueError, ValueError, ValueError) as exp:
        converter.structure(w, Worker)
    assert len(exp.value.exceptions) == 5
    for e in exp.value.exceptions:
        assert str(e) == "falsy values are not allowed"


@pytest.mark.parametrize("name", ["bob", None])
@pytest.mark.parametrize("email", ["bob@email.com", None])
@pytest.mark.parametrize("slug", ["bo-b", None])
@pytest.mark.parametrize("website", ["https://bob.com", None])
@pytest.mark.parametrize("unique_id", [uuid.uuid4(), None])
def test_structure_nullable(name, email, slug, website, unique_id):
    w = {
        "name": name,
        "email": email,
        "slug": slug,
        "website": website,
        "unique_id": unique_id,
    }

    structure = converter.structure(w, WorkerNullable)
    obj = WorkerNullable(**w)

    assert structure == obj

    assert structure.name is None or isinstance(structure.name, str)
    assert structure.email is None or isinstance(structure.email, str)
    assert structure.slug is None or isinstance(structure.slug, str)
    assert structure.website is None or isinstance(structure.website, str)
    assert structure.unique_id is None or isinstance(structure.unique_id, uuid.UUID)


def test_unstructure():
    w = {
        "name": "bob",
        "email": "bob@email.com",
        "slug": "bo-b",
        "website": "https://bob.com",
        "unique_id": uuid.uuid4(),
    }

    structure = converter.structure(w, Worker)

    unstructure = converter.unstructure(structure)

    assert unstructure == w


@pytest.mark.parametrize("name", ["bob", None])
@pytest.mark.parametrize("email", ["bob@email.com", None])
@pytest.mark.parametrize("slug", ["bo-b", None])
@pytest.mark.parametrize("website", ["https://bob.com", None])
@pytest.mark.parametrize("unique_id", [uuid.uuid4(), None])
def test_unstructure_nullable(name, email, slug, website, unique_id):
    w = {
        "name": name,
        "email": email,
        "slug": slug,
        "website": website,
        "unique_id": unique_id,
    }

    structure = converter.structure(w, WorkerNullable)

    unstructure = converter.unstructure(structure)

    assert unstructure == w

    assert unstructure["name"] is None or isinstance(structure.name, str)
    assert unstructure["email"] is None or isinstance(structure.email, str)
    assert unstructure["slug"] is None or isinstance(structure.slug, str)
    assert unstructure["website"] is None or isinstance(structure.website, str)
    assert unstructure["unique_id"] is None or isinstance(structure.unique_id, uuid.UUID)


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (tomlkit_serializer, tomlkit.dumps),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_dumps(converter, dumps):
    w = {
        "name": "bob",
        "email": "bob@email.com",
        "slug": "bo-b",
        "website": "https://bob.com",
        "unique_id": str(uuid.uuid4()),
    }
    if converter is cbor2_serializer or converter is bson_serializer:
        w["unique_id"] = uuid.uuid4()  # pyright: ignore[reportArgumentType]

    structure = converter.structure(w, Worker)

    dump = converter.dumps(structure)

    if converter is bson_serializer:
        w["unique_id"] = bson.Binary.from_uuid(w["unique_id"])

    assert dump == dumps(w)


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_loads(converter, dumps):
    w = {
        "name": "bob",
        "email": "bob@email.com",
        "slug": "bo-b",
        "website": "https://bob.com",
        "unique_id": str(uuid.uuid4()),
    }
    if converter is cbor2_serializer or converter is bson_serializer:
        w["unique_id"] = uuid.uuid4()  # pyright: ignore[reportArgumentType]
        if converter is bson_serializer:
            w["unique_id"] = bson.Binary.from_uuid(w["unique_id"])  # pyright: ignore[reportArgumentType]

    dump = dumps(w)

    x = converter.loads(dump, Worker)

    assert x == converter.structure(w, Worker)
    assert isinstance(x.unique_id, uuid.UUID)


@pytest.mark.parametrize("name", ["bob", None])
@pytest.mark.parametrize("email", ["bob@email.com", None])
@pytest.mark.parametrize("slug", ["bo-b", None])
@pytest.mark.parametrize("website", ["https://bob.com", None])
@pytest.mark.parametrize("unique_id", [uuid.uuid4(), None])
@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_loads_null(converter, dumps, name, email, slug, website, unique_id):
    w = {
        "name": name,
        "email": email,
        "slug": slug,
        "website": website,
        "unique_id": str(unique_id) if unique_id else None,
    }
    if converter is cbor2_serializer:
        w["unique_id"] = unique_id  # pyright: ignore[reportArgumentType]
    if converter is bson_serializer and unique_id:
        w["unique_id"] = bson.Binary.from_uuid(unique_id)

    dump = dumps(w)

    x = converter.loads(dump, WorkerNullable)

    assert x == converter.structure(w, WorkerNullable)


@pytest.mark.parametrize(
    "converter",
    [
        (bson_serializer),
        (cbor2_serializer),
        (json_serializer),
        (msgpack_serializer),
        (msgspec_serializer),
        (orjson_serializer),
        (pyyaml_serializer),
        (tomlkit_serializer),
        (ujson_serializer),
    ],
)
def test_dump_then_load(converter):
    w = {
        "name": "bob",
        "email": "bob@email.com",
        "slug": "bo-b",
        "website": "https://bob.com",
        "unique_id": str(uuid.uuid4()),
    }
    if converter is cbor2_serializer or converter is bson_serializer:
        w["unique_id"] = uuid.uuid4()  # pyright: ignore[reportArgumentType]
        if converter is bson_serializer:
            w["unique_id"] = bson.Binary.from_uuid(w["unique_id"])  # pyright: ignore[reportArgumentType]

    structure = converter.structure(w, Worker)

    dump = converter.dumps(structure)
    load = converter.loads(dump, dict)

    assert load == w


@pytest.mark.parametrize("name", ["bob", None])
@pytest.mark.parametrize("email", ["bob@email.com", None])
@pytest.mark.parametrize("slug", ["bo-b", None])
@pytest.mark.parametrize("website", ["https://bob.com", None])
@pytest.mark.parametrize("unique_id", [uuid.uuid4(), None])
@pytest.mark.parametrize(
    "converter",
    [
        (bson_serializer),
        (cbor2_serializer),
        (json_serializer),
        (msgpack_serializer),
        (msgspec_serializer),
        (orjson_serializer),
        (pyyaml_serializer),
        (ujson_serializer),
    ],
)
def test_dump_then_load_null(converter, name, email, slug, website, unique_id):
    w = {
        "name": name,
        "email": email,
        "slug": slug,
        "website": website,
        "unique_id": str(unique_id) if unique_id else None,
    }
    if converter is cbor2_serializer or converter is bson_serializer:
        w["unique_id"] = uuid.uuid4()  # pyright: ignore[reportArgumentType]
        if converter is bson_serializer:
            w["unique_id"] = bson.Binary.from_uuid(w["unique_id"])  # pyright: ignore[reportArgumentType]

    structure = converter.structure(w, WorkerNullable)

    dump = converter.dumps(structure)
    x = converter.loads(dump, WorkerNullable)

    assert x == converter.structure(w, WorkerNullable)



from django.core.files.uploadedfile import UploadedFile
from django.http import HttpResponseBadRequest, HttpResponseBase
from django.http.request import HttpRequest
from django.http.response import HttpResponse

import attrs

from django_cattrs_fields.converters import converter
from django_cattrs_fields.converters.json import serializer as json_serializer
from django_cattrs_fields.fields.files import FileField

from .models import Book


@attrs.define
class BookData:
    pdf: FileField


def view(request: HttpRequest) -> HttpResponseBase:
    if request.method == "POST":
        exp = None
        try:
            structured_data = converter.structure(
                {**request.POST.dict(), **request.FILES.dict()}, BookData
            )
        except* (ValueError, KeyError) as e:
            exp = e

        if exp:
            return HttpResponseBadRequest("no file")

        if not isinstance(structured_data.pdf, UploadedFile):
            return HttpResponseBadRequest("not instance of UploadedFile")

        data = converter.unstructure(structured_data)

        if not isinstance(data["pdf"], UploadedFile):
            return HttpResponseBadRequest("not instance of UploadedFile after unstructure")

        return HttpResponse("done")

    b = Book.objects.last()
    data = json_serializer.dumps(converter.structure({"pdf": b.pdf}, BookData))
    return HttpResponse(data)



from attrs import define

from django_cattrs_fields.converters import converter
from django_cattrs_fields.fields import (
    CharField,
    Empty,
    EmptyField,
    IntegerField,
)


@define
class WorkerPatch:
    age: IntegerField
    name: CharField | EmptyField = Empty


def test_structure():
    w = {"name": "bob", "age": 32}

    struct = converter.structure(w, WorkerPatch)

    obj = WorkerPatch(**w)

    assert struct == obj

    assert isinstance(struct.name, str)
    assert isinstance(struct.age, int)

    w = {"age": 32}

    struct = converter.structure(w, WorkerPatch)

    obj = WorkerPatch(**w)

    assert struct == obj

    assert isinstance(struct.name, EmptyField)
    assert isinstance(struct.age, int)


def test_unstructure():
    w = {"name": "bob", "age": 32}

    struct = converter.structure(w, WorkerPatch)

    unstruct = converter.unstructure(struct)

    assert unstruct == w

    w = {"age": 32}

    struct = converter.structure(w, WorkerPatch)

    unstruct = converter.unstructure(struct)

    assert unstruct == w



import math
from decimal import Decimal, DecimalException
from typing import get_args

from django.core import validators
from django.core.exceptions import ValidationError
from django.utils.regex_helper import _lazy_re_compile

from django_cattrs_fields.fields import (
    DecimalField,
    FloatField,
    IntegerField,
)
from django_cattrs_fields.validators import forbid_falsy_values_validator


# Integer hooks


def integer_structure(val, _) -> IntegerField:
    forbid_falsy_values_validator(val)
    return val


def integer_structure_nullable(val, _) -> IntegerField | None:
    if val is None:
        return None
    return integer_structure(val, _)


# django.forms.fields.IntegerField.re_decimal
re_decimal = _lazy_re_compile(r"\.0*\s*$")


def integer_unstructure(val: IntegerField | None) -> int | None:
    if val is None:
        return None

    try:
        v = int(re_decimal.sub("", str(val)))
    except (ValueError, TypeError) as e:
        raise ValueError from e

    return v


# Decimal hooks


def decimal_structure(val: str | Decimal, _, max_digits=None, decimal_places=None) -> DecimalField:
    forbid_falsy_values_validator(val)
    try:
        value = Decimal(val)
    except DecimalException:
        raise ValueError("Enter a number.")

    try:
        validators.DecimalValidator(max_digits=max_digits, decimal_places=decimal_places)(value)
    except ValidationError as e:
        raise ValueError(e.message)

    return value


def decimal_structure_annotated(val: str | Decimal, type) -> DecimalField:
    annotation = get_args(type)
    if len(annotation) > 1:
        max_digits: int | None = getattr(annotation[1], "decimal_max_digits", None)
        decimal_places: int | None = getattr(annotation[1], "decimal_places", None)
    else:
        max_digits = decimal_places = None

    return decimal_structure(val, type, max_digits=max_digits, decimal_places=decimal_places)


def decimal_structure_nullable(val, type) -> DecimalField | None:
    if val is None:
        return None
    return decimal_structure(val, type)


def decimal_unstructure(val: DecimalField | None) -> Decimal | None:
    if not val:
        return None
    return Decimal(val)


def decimal_unstructure_str(val: DecimalField | None) -> str | None:
    """some serializers can't handle decimals, convert to string in those cases"""
    if not val:
        return None
    return str(val.normalize())


# Float hooks


def float_structure(val, _) -> FloatField:
    forbid_falsy_values_validator(val)
    try:
        val = float(val)
    except (ValueError, TypeError) as e:
        raise ValueError from e

    if not math.isfinite(val):
        raise ValueError("infinite values are not supported.")

    return val


def float_structure_nullable(val, _) -> FloatField | None:
    if val is None:
        return None
    return float_structure(val, _)


def float_unstructure(val: FloatField | None) -> float | None:
    if val is None:
        return None
    try:
        val = float(val)  # pyright: ignore[reportAssignmentType]
    except (ValueError, TypeError) as e:
        raise ValueError from e
    return val



# Generated by Django 6.0 on 2026-01-06 13:05

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Book",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("pdf", models.FileField(upload_to="media/", null=True)),
            ],
        ),
    ]



from typing import Union, get_args

from attrs import has
from cattrs._compat import is_annotated
from cattrs.converters import Converter

from django.conf import settings

from django_cattrs_fields.fields import (
    BooleanField,
    CharField,
    DateField,
    DateTimeField,
    DecimalField,
    EmailField,
    EmptyField,
    FloatField,
    IntegerField,
    SlugField,
    TimeField,
    URLField,
    UUIDField,
)
from django_cattrs_fields.fields.files import FileField
from django_cattrs_fields.hooks import (
    boolean_structure,
    boolean_structure_nullable,
    boolean_unstructure,
    char_structure,
    char_structure_nullable,
    char_unstructure,
    date_structure,
    date_structure_nullable,
    date_unstructure,
    datetime_structure,
    datetime_structure_nullable,
    datetime_unstructure,
    decimal_structure,
    decimal_structure_annotated,
    decimal_structure_nullable,
    decimal_unstructure,
    email_structure,
    email_structure_nullable,
    email_unstructure,
    file_structure,
    file_structure_nullable,
    file_unstructure,
    float_structure,
    float_structure_nullable,
    float_unstructure,
    integer_structure,
    integer_structure_nullable,
    integer_unstructure,
    slug_structure,
    slug_structure_nullable,
    slug_unstructure,
    time_structure,
    time_structure_nullable,
    time_unstructure,
    url_structure,
    url_structure_nullable,
    url_unstructure,
    uuid_structure,
    uuid_structure_nullable,
    uuid_unstructure,
)
from django_cattrs_fields.hooks.empty_hooks import (
    empty_bool_structure,
    empty_bool_structure_nullable,
    empty_bool_unstructure,
    empty_char_structure,
    empty_char_structure_nullable,
    empty_char_unstructure,
    empty_date_structure,
    empty_date_structure_nullable,
    empty_date_unstructure,
    empty_datetime_structure,
    empty_datetime_structure_nullable,
    empty_datetime_unstructure,
    empty_decimal_structure,
    empty_decimal_structure_nullable,
    empty_decimal_unstructure,
    empty_email_structure,
    empty_email_structure_nullable,
    empty_email_unstructure,
    empty_file_structure,
    empty_file_structure_nullable,
    empty_file_unstructure,
    empty_float_structure,
    empty_float_structure_nullable,
    empty_float_unstructure,
    empty_integer_structure,
    empty_integer_structure_nullable,
    empty_integer_unstructure,
    empty_slug_structure,
    empty_slug_structure_nullable,
    empty_slug_unstructure,
    empty_structure,
    empty_time_structure,
    empty_time_structure_nullable,
    empty_time_unstructure,
    empty_unstructure,
    empty_url_structure,
    empty_url_structure_nullable,
    empty_url_unstructure,
    empty_uuid_structure,
    empty_uuid_structure_nullable,
    empty_uuid_unstructure,
    skip_empty,
)


def register_structure_hooks(converter: Converter):
    converter.register_structure_hook(BooleanField, boolean_structure)
    converter.register_structure_hook(CharField, char_structure)
    converter.register_structure_hook(DateField, date_structure)
    converter.register_structure_hook(DateTimeField, datetime_structure)
    converter.register_structure_hook_func(
        lambda t: is_annotated(t) and get_args(t)[0] is DecimalField, decimal_structure_annotated
    )
    converter.register_structure_hook(DecimalField, decimal_structure)
    converter.register_structure_hook(EmailField, email_structure)
    converter.register_structure_hook(EmptyField, empty_structure)
    converter.register_structure_hook(FloatField, float_structure)
    converter.register_structure_hook(IntegerField, integer_structure)
    converter.register_structure_hook(SlugField, slug_structure)
    converter.register_structure_hook(TimeField, time_structure)
    converter.register_structure_hook(URLField, url_structure)
    converter.register_structure_hook(UUIDField, uuid_structure)

    # Union types
    converter.register_structure_hook(Union[BooleanField, None], boolean_structure_nullable)
    converter.register_structure_hook(Union[CharField, None], char_structure_nullable)
    converter.register_structure_hook(Union[DateField, None], date_structure_nullable)
    converter.register_structure_hook(Union[DecimalField, None], decimal_structure_nullable)
    converter.register_structure_hook(Union[DateTimeField, None], datetime_structure_nullable)
    converter.register_structure_hook(Union[EmailField, None], email_structure_nullable)
    converter.register_structure_hook(Union[FloatField, None], float_structure_nullable)
    converter.register_structure_hook(Union[IntegerField, None], integer_structure_nullable)
    converter.register_structure_hook(Union[SlugField, None], slug_structure_nullable)
    converter.register_structure_hook(Union[TimeField, None], time_structure_nullable)
    converter.register_structure_hook(Union[URLField, None], url_structure_nullable)
    converter.register_structure_hook(Union[UUIDField, None], uuid_structure_nullable)

    # Empty Unions
    converter.register_structure_hook(Union[BooleanField, EmptyField], empty_bool_structure)
    converter.register_structure_hook(Union[CharField, EmptyField], empty_char_structure)
    converter.register_structure_hook(Union[EmptyField, EmptyField], empty_email_structure)
    converter.register_structure_hook(Union[SlugField, EmptyField], empty_slug_structure)
    converter.register_structure_hook(Union[URLField, EmptyField], empty_url_structure)
    converter.register_structure_hook(Union[UUIDField, EmptyField], empty_uuid_structure)
    converter.register_structure_hook(Union[IntegerField, EmptyField], empty_integer_structure)
    converter.register_structure_hook(Union[DecimalField, EmptyField], empty_decimal_structure)
    converter.register_structure_hook(Union[FloatField, EmptyField], empty_float_structure)
    converter.register_structure_hook(Union[DateField, EmptyField], empty_date_structure)
    converter.register_structure_hook(Union[DateTimeField, EmptyField], empty_datetime_structure)
    converter.register_structure_hook(Union[TimeField, EmptyField], empty_time_structure)

    converter.register_structure_hook(
        Union[BooleanField, EmptyField], empty_bool_structure_nullable
    )
    converter.register_structure_hook(
        Union[CharField, EmptyField, None], empty_char_structure_nullable
    )
    converter.register_structure_hook(
        Union[EmptyField, EmptyField, None], empty_email_structure_nullable
    )
    converter.register_structure_hook(
        Union[SlugField, EmptyField, None], empty_slug_structure_nullable
    )
    converter.register_structure_hook(
        Union[URLField, EmptyField, None], empty_url_structure_nullable
    )
    converter.register_structure_hook(
        Union[UUIDField, EmptyField, None], empty_uuid_structure_nullable
    )
    converter.register_structure_hook(
        Union[IntegerField, EmptyField, None], empty_integer_structure_nullable
    )
    converter.register_structure_hook(
        Union[DecimalField, EmptyField, None], empty_decimal_structure_nullable
    )
    converter.register_structure_hook(
        Union[FloatField, EmptyField, None], empty_float_structure_nullable
    )
    converter.register_structure_hook(
        Union[DateField, EmptyField, None], empty_date_structure_nullable
    )
    converter.register_structure_hook(
        Union[DateTimeField, EmptyField, None], empty_datetime_structure_nullable
    )
    converter.register_structure_hook(
        Union[TimeField, EmptyField, None], empty_time_structure_nullable
    )

    # File

    if getattr(settings, "DCF_FILE_HOOKS", True):
        converter.register_structure_hook(FileField, file_structure)
        converter.register_structure_hook(Union[FileField, None], file_structure_nullable)
        converter.register_structure_hook(Union[FileField, EmptyField], empty_file_structure)
        converter.register_structure_hook(
            Union[FileField, EmptyField, None], empty_file_structure_nullable
        )


def register_unstructure_hooks(converter: Converter):
    converter.register_unstructure_hook_factory(has, skip_empty)

    converter.register_unstructure_hook(BooleanField, boolean_unstructure)
    converter.register_unstructure_hook(CharField, char_unstructure)
    converter.register_unstructure_hook(EmailField, email_unstructure)
    converter.register_unstructure_hook(EmptyField, empty_unstructure)
    converter.register_unstructure_hook(FloatField, float_unstructure)
    converter.register_unstructure_hook(IntegerField, integer_unstructure)
    converter.register_unstructure_hook(SlugField, slug_unstructure)
    converter.register_unstructure_hook(URLField, url_unstructure)

    # Union types
    converter.register_unstructure_hook(Union[BooleanField, None], boolean_unstructure)
    converter.register_unstructure_hook(Union[CharField, None], char_unstructure)
    converter.register_unstructure_hook(Union[EmailField, None], email_unstructure)
    converter.register_unstructure_hook(Union[FloatField, None], float_unstructure)
    converter.register_unstructure_hook(Union[IntegerField, None], integer_unstructure)
    converter.register_unstructure_hook(Union[SlugField, None], slug_unstructure)
    converter.register_unstructure_hook(Union[URLField, None], url_unstructure)

    # Empty Unions
    converter.register_unstructure_hook(Union[BooleanField, EmptyField], empty_bool_unstructure)
    converter.register_unstructure_hook(Union[CharField, EmptyField], empty_char_unstructure)
    converter.register_unstructure_hook(Union[EmailField, EmptyField], empty_email_unstructure)
    converter.register_unstructure_hook(Union[SlugField, EmptyField], empty_slug_unstructure)
    converter.register_unstructure_hook(Union[URLField, EmptyField], empty_url_unstructure)
    converter.register_unstructure_hook(Union[UUIDField, EmptyField], empty_uuid_unstructure)
    converter.register_unstructure_hook(Union[IntegerField, EmptyField], empty_integer_unstructure)
    converter.register_unstructure_hook(Union[DecimalField, EmptyField], empty_decimal_unstructure)
    converter.register_unstructure_hook(Union[FloatField, EmptyField], empty_float_unstructure)
    converter.register_unstructure_hook(Union[DateField, EmptyField], empty_date_unstructure)
    converter.register_unstructure_hook(
        Union[DateTimeField, EmptyField], empty_datetime_unstructure
    )
    converter.register_unstructure_hook(Union[TimeField, EmptyField], empty_time_unstructure)

    converter.register_unstructure_hook(
        Union[BooleanField, EmptyField, None], empty_bool_unstructure
    )
    converter.register_unstructure_hook(Union[CharField, EmptyField, None], empty_char_unstructure)
    converter.register_unstructure_hook(
        Union[EmailField, EmptyField, None], empty_email_unstructure
    )
    converter.register_unstructure_hook(Union[SlugField, EmptyField, None], empty_slug_unstructure)
    converter.register_unstructure_hook(Union[URLField, EmptyField, None], empty_url_unstructure)
    converter.register_unstructure_hook(Union[UUIDField, EmptyField, None], empty_uuid_unstructure)
    converter.register_unstructure_hook(
        Union[IntegerField, EmptyField, None], empty_integer_unstructure
    )
    converter.register_unstructure_hook(
        Union[DecimalField, EmptyField, None], empty_decimal_unstructure
    )
    converter.register_unstructure_hook(
        Union[FloatField, EmptyField, None], empty_float_unstructure
    )
    converter.register_unstructure_hook(Union[DateField, EmptyField, None], empty_date_unstructure)
    converter.register_unstructure_hook(
        Union[DateTimeField, EmptyField], empty_datetime_unstructure
    )
    converter.register_unstructure_hook(Union[TimeField, EmptyField], empty_time_unstructure)

    # File

    if getattr(settings, "DCF_FILE_HOOKS", True):
        converter.register_unstructure_hook(FileField, file_unstructure)
        converter.register_unstructure_hook(Union[FileField, None], file_unstructure)
        converter.register_unstructure_hook(Union[FileField, EmptyField], empty_file_unstructure)
        converter.register_unstructure_hook(
            Union[FileField, EmptyField, None], empty_file_unstructure
        )


def register_uuid_unstructure_hooks(converter: Converter):
    converter.register_unstructure_hook(UUIDField, uuid_unstructure)
    converter.register_unstructure_hook(Union[UUIDField, None], uuid_unstructure)


def register_date_unstructure_hooks(converter: Converter):
    converter.register_unstructure_hook(DateField, date_unstructure)
    converter.register_unstructure_hook(Union[DateField, None], date_unstructure)


def register_datetime_unstructure_hooks(converter: Converter):
    converter.register_unstructure_hook(DateTimeField, datetime_unstructure)
    converter.register_unstructure_hook(Union[DateTimeField, None], datetime_unstructure)


def register_decimal_unstructure_hooks(converter: Converter):
    converter.register_unstructure_hook(DecimalField, decimal_unstructure)
    converter.register_unstructure_hook(Union[DecimalField, None], decimal_unstructure)


def register_time_unstructure_hooks(converter: Converter):
    converter.register_unstructure_hook(TimeField, time_unstructure)
    converter.register_unstructure_hook(Union[TimeField, None], time_unstructure)


def register_all_unstructure_hooks(converter: Converter):
    register_unstructure_hooks(converter)
    register_uuid_unstructure_hooks(converter)
    register_date_unstructure_hooks(converter)
    register_datetime_unstructure_hooks(converter)
    register_decimal_unstructure_hooks(converter)
    register_time_unstructure_hooks(converter)



from .bool_hooks import *
from .char_hooks import *
from .date_hooks import *
from .empty_hooks import (
    empty_bool_structure,
    empty_bool_structure_nullable,
    empty_bool_unstructure,
    empty_char_structure,
    empty_char_structure_nullable,
    empty_char_unstructure,
    empty_date_structure,
    empty_date_structure_nullable,
    empty_date_unstructure,
    empty_datetime_structure,
    empty_datetime_structure_nullable,
    empty_datetime_unstructure,
    empty_decimal_structure,
    empty_decimal_structure_nullable,
    empty_decimal_unstructure,
    empty_email_structure,
    empty_email_structure_nullable,
    empty_email_unstructure,
    empty_file_structure,
    empty_file_structure_nullable,
    empty_file_unstructure,
    empty_float_structure,
    empty_float_structure_nullable,
    empty_float_unstructure,
    empty_integer_structure,
    empty_integer_structure_nullable,
    empty_integer_unstructure,
    empty_slug_structure,
    empty_slug_structure_nullable,
    empty_slug_unstructure,
    empty_structure,
    empty_time_structure,
    empty_time_structure_nullable,
    empty_time_unstructure,
    empty_unstructure,
    empty_url_structure,
    empty_url_structure_nullable,
    empty_url_unstructure,
    empty_uuid_structure,
    empty_uuid_structure_nullable,
    empty_uuid_unstructure,
    skip_empty,
)
from .file_hooks import *
from .number_hooks import *

# ruff: noqa: F405

__all__ = (
    "boolean_structure",
    "boolean_structure_nullable",
    "boolean_unstructure",
    "char_structure",
    "char_structure_nullable",
    "char_unstructure",
    "date_structure",
    "date_structure_nullable",
    "date_unstructure",
    "datetime_structure",
    "datetime_structure_nullable",
    "datetime_unstructure",
    "decimal_structure",
    "decimal_structure_annotated",
    "decimal_structure_nullable",
    "decimal_unstructure",
    "decimal_unstructure_str",
    "email_structure",
    "email_structure_nullable",
    "email_unstructure",
    "empty_bool_structure",
    "empty_bool_structure_nullable",
    "empty_bool_unstructure",
    "empty_char_structure",
    "empty_char_structure_nullable",
    "empty_char_unstructure",
    "empty_date_structure",
    "empty_date_structure_nullable",
    "empty_date_unstructure",
    "empty_datetime_structure",
    "empty_datetime_structure_nullable",
    "empty_datetime_unstructure",
    "empty_decimal_structure",
    "empty_decimal_structure_nullable",
    "empty_decimal_unstructure",
    "empty_email_structure",
    "empty_email_structure_nullable",
    "empty_email_unstructure",
    "empty_file_structure",
    "empty_file_structure_nullable",
    "empty_file_unstructure",
    "empty_float_structure",
    "empty_float_structure_nullable",
    "empty_float_unstructure",
    "empty_integer_structure",
    "empty_integer_structure_nullable",
    "empty_integer_unstructure",
    "empty_slug_structure",
    "empty_slug_structure_nullable",
    "empty_slug_unstructure",
    "empty_structure",
    "empty_time_structure",
    "empty_time_structure_nullable",
    "empty_time_unstructure",
    "empty_unstructure",
    "empty_url_structure",
    "empty_url_structure_nullable",
    "empty_url_unstructure",
    "empty_uuid_structure",
    "empty_uuid_structure_nullable",
    "empty_uuid_unstructure",
    "file_structure",
    "file_structure_nullable",
    "file_unstructure",
    "float_structure",
    "float_structure_nullable",
    "float_unstructure",
    "integer_structure",
    "integer_structure_nullable",
    "integer_unstructure",
    "skip_empty",
    "slug_structure",
    "slug_structure_nullable",
    "slug_unstructure",
    "url_structure",
    "url_structure_nullable",
    "url_unstructure",
    "uuid_structure",
    "uuid_structure_nullable",
    "uuid_unstructure",
)



from django.conf import settings

from cattrs.preconf.tomlkit import make_converter

from django_cattrs_fields.fields import DecimalField, UUIDField
from django_cattrs_fields.hooks.number_hooks import decimal_unstructure_str

from .register_hooks import (
    register_structure_hooks,
    register_unstructure_hooks,
    register_datetime_unstructure_hooks,
    register_date_unstructure_hooks,
    register_time_unstructure_hooks,
)

serializer = make_converter()

register_structure_hooks(serializer)

register_unstructure_hooks(serializer)
register_date_unstructure_hooks(serializer)
register_datetime_unstructure_hooks(serializer)
register_time_unstructure_hooks(serializer)


if getattr(settings, "DCF_SERIALIZER_HOOKS", True):
    serializer.register_unstructure_hook(UUIDField, lambda x: str(x))
    serializer.register_unstructure_hook(DecimalField, decimal_unstructure_str)

__all__ = ("serializer",)



import json
from decimal import Decimal
from typing import Annotated

import pytest

from attrs import define

import bson
import cbor2
import msgpack
import orjson
import ujson
import yaml
import tomlkit

from msgspec import json as msgspec_json

from django_cattrs_fields.converters import converter
from django_cattrs_fields.converters.bson import serializer as bson_serializer
from django_cattrs_fields.converters.cbor2 import serializer as cbor2_serializer
from django_cattrs_fields.converters.json import serializer as json_serializer
from django_cattrs_fields.converters.msgpack import serializer as msgpack_serializer
from django_cattrs_fields.converters.msgspec import serializer as msgspec_serializer
from django_cattrs_fields.converters.orjson import serializer as orjson_serializer
from django_cattrs_fields.converters.pyyaml import serializer as pyyaml_serializer
from django_cattrs_fields.converters.tomlkit import serializer as tomlkit_serializer
from django_cattrs_fields.converters.ujson import serializer as ujson_serializer
from django_cattrs_fields.fields import DecimalField, IntegerField, FloatField, Params


@define
class PeopleNumbers:
    age: IntegerField
    salary: FloatField
    accurate_salary: DecimalField


@define
class PeopleNumbersAnnotated:
    age: IntegerField
    salary: FloatField
    accurate_salary: Annotated[DecimalField, Params(decimal_max_digits=5, decimal_places=3)]


@define
class PeopleNumbersNullable:
    age: IntegerField | None
    salary: FloatField | None
    accurate_salary: DecimalField | None


@define
class PeopleNumbersNullableAnnotated:
    age: IntegerField | None
    salary: FloatField | None
    accurate_salary: Annotated[DecimalField, Params(decimal_max_digits=5, decimal_places=3)] | None


def test_structure():
    pn = {"age": 25, "salary": 100.5, "accurate_salary": "10.5"}

    structure = converter.structure(pn, PeopleNumbers)
    pn["accurate_salary"] = Decimal(pn["accurate_salary"])
    obj = PeopleNumbers(**pn)

    assert structure == obj
    assert isinstance(structure.age, int)
    assert isinstance(structure.salary, float)
    assert isinstance(structure.accurate_salary, Decimal)


def test_structure_annotated():
    pn = {"age": 25, "salary": 100.5, "accurate_salary": "10.5"}

    structure = converter.structure(pn, PeopleNumbersAnnotated)
    pn["accurate_salary"] = Decimal(pn["accurate_salary"])
    obj = PeopleNumbersAnnotated(**pn)

    assert structure == obj
    assert isinstance(structure.age, int)
    assert isinstance(structure.salary, float)
    assert isinstance(structure.accurate_salary, Decimal)

    pn["accurate_salary"] = "100.12"
    with pytest.RaisesGroup(ValueError):
        converter.structure(pn, PeopleNumbersAnnotated)

    pn["accurate_salary"] = "10.1234"
    with pytest.RaisesGroup(ValueError):
        converter.structure(pn, PeopleNumbersAnnotated)


@pytest.mark.parametrize(
    "age, salary, accurate_salary", [(None, 43.1, None), (11, None, None), (None, None, "11.4")]
)
def test_structure_nullable(age, salary, accurate_salary):
    pn = {"age": age, "salary": salary, "accurate_salary": accurate_salary}

    structure = converter.structure(pn, PeopleNumbersNullable)
    if accurate_salary:
        pn["accurate_salary"] = Decimal(accurate_salary)
    obj = PeopleNumbersNullable(**pn)

    assert structure == obj

    assert isinstance(structure.age, int) or structure.age is None
    assert structure.salary is salary or isinstance(structure.salary, float)
    assert isinstance(structure.accurate_salary, Decimal) or structure.accurate_salary is None


@pytest.mark.parametrize(
    "age, salary, accurate_salary", [(None, 43.1, None), (11, None, None), (None, None, "11.4")]
)
def test_structure_nullable_annotated(age, salary, accurate_salary):
    pn = {"age": age, "salary": salary, "accurate_salary": accurate_salary}

    structure = converter.structure(pn, PeopleNumbersNullableAnnotated)
    if accurate_salary:
        pn["accurate_salary"] = Decimal(accurate_salary)
    obj = PeopleNumbersNullableAnnotated(**pn)

    assert structure == obj

    assert isinstance(structure.age, int) or structure.age is None
    assert structure.salary is salary or isinstance(structure.salary, float)
    assert isinstance(structure.accurate_salary, Decimal) or structure.accurate_salary is None

    pn["accurate_salary"] = "100.12"
    with pytest.RaisesGroup(ValueError):
        converter.structure(pn, PeopleNumbersNullableAnnotated)

    pn["accurate_salary"] = "10.1234"
    with pytest.RaisesGroup(ValueError):
        converter.structure(pn, PeopleNumbersNullableAnnotated)


@pytest.mark.parametrize(
    "age, salary, accurate_salary", [(None, 43.1, "11.4"), (11, 12.1, None), (11.3, None, "11.4")]
)
def test_structure_null_with_not_nullable_raises(age, salary, accurate_salary):
    pn = {"age": age, "salary": salary, "accurate_salary": accurate_salary}
    with pytest.RaisesGroup(ValueError):
        converter.structure(pn, PeopleNumbers)


def test_unstructure():
    pn = {"age": 25, "salary": 100.5, "accurate_salary": "11.4"}
    structure = converter.structure(pn, PeopleNumbers)

    unstructure = converter.unstructure(structure)

    pn["accurate_salary"] = Decimal(pn["accurate_salary"])
    assert unstructure == pn

    assert isinstance(unstructure["age"], int)
    assert isinstance(unstructure["salary"], float)
    assert isinstance(unstructure["accurate_salary"], Decimal)


def test_unstructure_annotated():
    pn = {"age": 25, "salary": 100.5, "accurate_salary": "11.4"}
    structure = converter.structure(pn, PeopleNumbersAnnotated)

    unstructure = converter.unstructure(structure)

    pn["accurate_salary"] = Decimal(pn["accurate_salary"])
    assert unstructure == pn

    assert isinstance(unstructure["age"], int)
    assert isinstance(unstructure["salary"], float)
    assert isinstance(unstructure["accurate_salary"], Decimal)


@pytest.mark.parametrize(
    "age, salary, accurate_salary", [(None, 43.1, None), (11, None, None), (None, None, "43.1")]
)
def test_unstructure_nullable(age, salary, accurate_salary):
    pn = {"age": age, "salary": salary, "accurate_salary": accurate_salary}
    structure = converter.structure(pn, PeopleNumbersNullable)
    unstructure = converter.unstructure(structure)

    if accurate_salary:
        accurate_salary = Decimal(accurate_salary)
        pn["accurate_salary"] = accurate_salary
    assert unstructure == pn

    assert isinstance(unstructure["age"], int) or unstructure["age"] is None
    assert unstructure["salary"] is salary or unstructure["salary"] == salary
    assert (
        unstructure["accurate_salary"] is accurate_salary
        or unstructure["accurate_salary"] == accurate_salary
    )


@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (tomlkit_serializer, tomlkit.dumps),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_dumps(converter, dumps):
    pn = {"age": 25, "salary": 100.5, "accurate_salary": "11.1"}
    structure = converter.structure(pn, PeopleNumbers)
    if converter in {cbor2_serializer}:
        pn["accurate_salary"] = Decimal(pn["accurate_salary"])

    dump = converter.dumps(structure)

    assert dump == dumps(pn)


@pytest.mark.parametrize(
    "age, salary, accurate_salary", [(None, 43.1, None), (11, None, None), (None, None, "43.13")]
)
@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_dumps_nullable(converter, dumps, age, salary, accurate_salary):
    pn = {"age": age, "salary": salary, "accurate_salary": accurate_salary}
    structure = converter.structure(pn, PeopleNumbersNullable)

    dump = converter.dumps(structure)

    if accurate_salary:
        if converter in {cbor2_serializer}:
            pn["accurate_salary"] = Decimal(accurate_salary)
    assert dump == dumps(pn)


@pytest.mark.parametrize("age, salary, accurate_salary", [(43, 43.1, "12.32"), (11, 11, 21)])
@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (tomlkit_serializer, tomlkit.dumps),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_loads(converter, dumps, age, salary, accurate_salary):
    pn = {"age": age, "salary": salary, "accurate_salary": accurate_salary}
    dump = dumps(pn)

    x = converter.loads(dump, PeopleNumbers)
    pn["accurate_salary"] = pn["accurate_salary"]

    pn["accurate_salary"] = Decimal(pn["accurate_salary"])
    assert x == PeopleNumbers(**pn)


@pytest.mark.parametrize(
    "age, salary, accurate_salary", [(None, 43.1, None), (11, None, None), (None, None, "13.11")]
)
@pytest.mark.parametrize(
    "converter, dumps",
    [
        (bson_serializer, bson.encode),
        (cbor2_serializer, cbor2.dumps),
        (json_serializer, json.dumps),
        (msgpack_serializer, msgpack.dumps),
        (msgspec_serializer, msgspec_json.encode),
        (orjson_serializer, orjson.dumps),
        (pyyaml_serializer, yaml.safe_dump),
        (ujson_serializer, ujson.dumps),
    ],
)
def test_loads_nullable(converter, dumps, age, salary, accurate_salary):
    pn = {"age": age, "salary": salary, "accurate_salary": accurate_salary}
    dump = dumps(pn)

    x = converter.loads(dump, PeopleNumbersNullable)

    if accurate_salary:
        pn["accurate_salary"] = Decimal(accurate_salary)
    assert x == PeopleNumbersNullable(**pn)
    assert x.salary is salary or x.salary == salary


@pytest.mark.parametrize(
    "converter",
    [
        (bson_serializer),
        (cbor2_serializer),
        (json_serializer),
        (msgpack_serializer),
        (msgspec_serializer),
        (orjson_serializer),
        (pyyaml_serializer),
        (tomlkit_serializer),
        (ujson_serializer),
    ],
)
def test_dump_then_load(converter):
    pn = {"age": 25, "salary": 100.5, "accurate_salary": "12.43"}
    structure = converter.structure(pn, PeopleNumbers)

    dump = converter.dumps(structure)
    load = converter.loads(dump, PeopleNumbers)

    if converter in {msgspec_serializer, cbor2_serializer}:
        pn["accurate_salary"] = Decimal(pn["accurate_salary"])
    assert converter.unstructure(load) == pn


@pytest.mark.parametrize(
    "age, salary, accurate_salary", [(None, 43.1, None), (11, None, None), (None, None, "12.11")]
)
@pytest.mark.parametrize(
    "converter",
    [
        (bson_serializer),
        (cbor2_serializer),
        (json_serializer),
        (msgpack_serializer),
        (msgspec_serializer),
        (orjson_serializer),
        (pyyaml_serializer),
        (ujson_serializer),
    ],
)
def test_dump_then_load_nullable(converter, age, salary, accurate_salary):
    pn = {"age": age, "salary": salary, "accurate_salary": accurate_salary}
    structure = converter.structure(pn, PeopleNumbersNullable)

    dump = converter.dumps(structure)
    load = converter.loads(dump, PeopleNumbersNullable)

    if converter in {msgspec_serializer, cbor2_serializer}:
        if accurate_salary:
            pn["accurate_salary"] = Decimal(accurate_salary)
    assert converter.unstructure(load) == pn



import pytest

from django.core.files.uploadedfile import SimpleUploadedFile

from django_cattrs_fields.converters.json import serializer as json_serializer

from tests.books.models import Book


@pytest.fixture
def simple_file():
    return SimpleUploadedFile(name="test_image.jpeg", content=b"wheeee", content_type="image/jpeg")


@pytest.fixture
def create_book(db):
    Book.objects.create(
        pdf=SimpleUploadedFile(name="test_image.jpeg", content=b"wheeee", content_type="image/jpeg")
    )


def test_upload_file_get_200(client, simple_file):
    result = client.post("", {"pdf": simple_file})
    assert result.status_code == 200


def test_post_empty(client):
    result = client.post("")
    assert result.status_code == 400
    assert result.text == "no file"


def test_get_file(client, create_book):
    result = client.get("")
    data = json_serializer.loads(result.text, dict)
    assert "pdf" in data
    assert isinstance(data["pdf"], str)



import uuid

from django.core import validators
from django.core.exceptions import ValidationError

from django_cattrs_fields.fields import (
    CharField,
    EmailField,
    SlugField,
    UUIDField,
    URLField,
)
from django_cattrs_fields.validators import (
    null_char_validator,
    char_field_validation,
    slug_field_validation,
)


__all__ = (
    "char_structure",
    "char_structure_nullable",
    "char_unstructure",
    "email_structure",
    "email_structure_nullable",
    "email_unstructure",
    "slug_structure",
    "slug_structure_nullable",
    "slug_unstructure",
    "uuid_structure",
    "uuid_structure_nullable",
    "uuid_unstructure",
    "url_structure",
    "url_structure_nullable",
    "url_unstructure",
)

# Char hooks


def char_structure(val, _) -> CharField:
    char_field_validation(val)
    return val


def char_structure_nullable(val, _) -> CharField | None:
    if val is None:
        return None
    return char_structure(val, _)


def char_unstructure(val: CharField | None) -> str | None:
    return val


# Email hooks


def email_structure(val, _) -> EmailField:
    char_field_validation(val)
    try:
        validators.validate_email(val)
    except ValidationError as e:
        raise ValueError(e.message)
    return val


def email_structure_nullable(val, _) -> EmailField | None:
    if val is None:
        return None

    return email_structure(val, _)


def email_unstructure(val: EmailField | None) -> str | None:
    return val


# Slug hooks


def slug_structure(val, _) -> SlugField:
    slug_field_validation(val)
    return val


def slug_structure_nullable(val, _) -> SlugField | None:
    if val is None:
        return None
    return val


def slug_unstructure(val: SlugField | None) -> str | None:
    return val


# UUID hooks


def uuid_structure(val, _) -> UUIDField:
    null_char_validator(val)

    if isinstance(val, uuid.UUID):
        return val
    return uuid.UUID(val)


def uuid_structure_nullable(val, _) -> UUIDField | None:
    if val is None:
        return None
    return uuid_structure(val, _)


def uuid_unstructure(val: UUIDField | None) -> uuid.UUID | None:
    return val


# URL hooks


def url_structure(val, _) -> URLField:
    char_field_validation(val)
    try:
        validators.URLValidator()(val)
    except ValidationError as e:
        raise ValueError(e.message)

    return val


def url_structure_nullable(val, _) -> URLField | None:
    if val is None:
        return None

    return url_structure(val, _)


def url_unstructure(val: URLField | None) -> str | None:
    return val



from django.core import validators
from django.core.exceptions import ValidationError


def forbid_falsy_values_validator(val):
    if not val:
        raise ValueError("falsy values are not allowed")


def null_char_validator(val):
    forbid_falsy_values_validator(val)
    try:
        validators.ProhibitNullCharactersValidator()(val)
    except ValidationError as e:
        raise ValueError(e.message)


def char_field_validation(val):
    null_char_validator(val)
    if not isinstance(val, str):
        raise ValueError("value should be a string object")


def slug_field_validation(val):
    char_field_validation(val)
    try:
        validators.validate_unicode_slug(val)
    except ValidationError as e:
        raise ValueError(e.message)



import datetime
from datetime import timezone, tzinfo

from django.conf import settings
from django.utils import timezone as dj_timezone

# these codes were copied from drf
# TODO: check the quality


def datetime_exists(dt: datetime.datetime):
    """Check if a datetime exists. Taken from: https://pytz-deprecation-shim.readthedocs.io/en/latest/migration.html"""
    # There are no non-existent times in UTC, and comparisons between
    # aware time zones always compare absolute times; if a datetime is
    # not equal to the same datetime represented in UTC, it is imaginary.
    return dt.astimezone(timezone.utc) == dt


def datetime_ambiguous(dt: datetime.datetime):
    """Check whether a datetime is ambiguous. Taken from: https://pytz-deprecation-shim.readthedocs.io/en/latest/migration.html"""
    # If a datetime exists and its UTC offset changes in response to
    # changing `fold`, it is ambiguous in the zone specified.
    return datetime_exists(dt) and (dt.replace(fold=not dt.fold).utcoffset() != dt.utcoffset())


def valid_datetime(dt: datetime.datetime):
    """Returns True if the datetime is not ambiguous or imaginary, False otherwise."""
    if isinstance(dt.tzinfo, tzinfo) and not datetime_ambiguous(dt):
        return True
    return False


def enforce_timezone(value: datetime.datetime) -> datetime.datetime:
    """
    When `self.default_timezone` is `None`, always return naive datetimes.
    When `self.default_timezone` is not `None`, always return aware datetimes.
    """
    field_timezone = dj_timezone.get_current_timezone() if settings.USE_TZ else None
    if field_timezone is not None:
        if dj_timezone.is_aware(value):
            try:
                return value.astimezone(field_timezone)
            except OverflowError:
                raise ValueError("Datetime value out of range")
        try:
            dt = dj_timezone.make_aware(value, field_timezone)
            # When the resulting datetime is a ZoneInfo instance, it won't necessarily
            # throw given an invalid datetime, so we need to specifically check.
            if not valid_datetime(dt):
                raise ValueError(f"Invalid datetime for the timezone {field_timezone}")
            return dt
        except Exception as e:
            raise ValueError from e
    elif (field_timezone is None) and dj_timezone.is_aware(value):
        return dj_timezone.make_naive(value, datetime.timezone.utc)
    return value



from typing import Union

from cattrs.preconf.cbor2 import make_converter
from django.conf import settings

from django_cattrs_fields.fields import TimeField
from django_cattrs_fields.hooks.date_hooks import time_unstructure_str

from .register_hooks import (
    register_date_unstructure_hooks,
    register_datetime_unstructure_hooks,
    register_decimal_unstructure_hooks,
    register_structure_hooks,
    register_unstructure_hooks,
    register_uuid_unstructure_hooks,
)

serializer = make_converter()

register_structure_hooks(serializer)
register_unstructure_hooks(serializer)
register_datetime_unstructure_hooks(serializer)
register_date_unstructure_hooks(serializer)
register_decimal_unstructure_hooks(serializer)
register_uuid_unstructure_hooks(serializer)

if getattr(settings, "DCF_SERIALIZER_HOOKS", True):
    serializer.register_unstructure_hook(TimeField, time_unstructure_str)
    serializer.register_unstructure_hook(Union[TimeField, None], time_unstructure_str)

__all__ = ("serializer",)
